@@ -15,11 +15,14 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, glob, sys, string
+import os, glob, sys
+import urllib
 import zipfile
 import locale
 
 import Image
+# lazygal has her own ImageFile class, so avoid trouble
+import ImageFile as PILImageFile
 import genshi
 
 import __init__
@@ -39,12 +42,19 @@ DEST_SHARED_DIRECTORY_NAME = 'shared'
 
 SOURCEDIR_CONFIGFILE = '.lazygal'
 
+THUMB_SIZE_NAME = 'thumb'
+
 
 class ImageOriginal(make.FileCopy):
 
-    def __init__(self, dir, source_image):
+    def __init__(self, dir, source_image, size_name=None):
         self.dir = dir
-        self.path = os.path.join(self.dir.path, source_image.filename)
+
+        dest_name = source_image.filename
+        if size_name:
+            dest_name = self.dir.album._add_size_qualifier(dest_name, size_name)
+
+        self.path = os.path.join(self.dir.path, dest_name)
         make.FileCopy.__init__(self, source_image.path, self.path)
 
     def build(self):
@@ -65,7 +75,7 @@ class ImageOtherSize(make.FileMakeObject):
         make.FileMakeObject.__init__(self, self.osize_path)
 
         self.size_name = size_name
-        if self.size_name == 'thumb':
+        if self.size_name == THUMB_SIZE_NAME:
             self.size = self.dir.album.thumb_size
         else:
             self.size = self.dir.album.browse_sizes[size_name]
@@ -73,7 +83,7 @@ class ImageOtherSize(make.FileMakeObject):
         self.add_dependency(self.source_image)
 
     def build(self):
-        self.dir.album.log("  RESIZE %s" % os.path.basename(self.osize_path),
+        self.dir.album.log(_("  RESIZE %s") % os.path.basename(self.osize_path),
                            'info')
         self.dir.album.log("(%s)" % self.osize_path)
 
@@ -86,7 +96,17 @@ class ImageOtherSize(make.FileMakeObject):
 
         im.thumbnail(self.size, Image.ANTIALIAS)
 
-        im.save(self.osize_path, quality = self.dir.album.quality)
+        calibrated = False
+        while not calibrated:
+            try:
+                im.save(self.osize_path, quality = self.dir.album.quality, **self.dir.album.save_options)
+            except IOError, e:
+                if str(e).startswith('encoder error'):
+                    PILImageFile.MAXBLOCK = 2 * PILImageFile.MAXBLOCK
+                    continue
+                else:
+                    raise
+            calibrated = True
 
 
 class WebalbumPicture(make.FileMakeObject):
@@ -103,7 +123,8 @@ class WebalbumPicture(make.FileMakeObject):
 
         # Use already generated thumbs for better performance (lighter to
         # rotate, etc.).
-        pics = map(lambda path: self.album._add_size_qualifier(path, 'thumb'),
+        pics = map(lambda path: self.album._add_size_qualifier(path,
+                                                               THUMB_SIZE_NAME),
                    lightdir.get_all_images_paths())
 
         for pic in pics:
@@ -111,7 +132,8 @@ class WebalbumPicture(make.FileMakeObject):
 
         if lightdir.album_picture:
             md_dirpic_thumb = self.album._add_size_qualifier(\
-                                           lightdir.album_picture, 'thumb')
+                                           lightdir.album_picture,
+                                           THUMB_SIZE_NAME)
             md_dirpic_thumb = os.path.join(lightdir.path, md_dirpic_thumb)
         else:
             md_dirpic_thumb = None
@@ -173,12 +195,14 @@ class WebalbumPage(make.FileMakeObject):
             link_vals = {}
             link_vals['link'] = self._add_size_qualifier(img.name + '.html',
                                                          self.size_name)
+            link_vals['link'] = self.url_quote(link_vals['link'])
             link_vals['thumb'] = self._add_size_qualifier(img.name\
                                                           + img.extension,
-                                                          'thumb')
+                                                          THUMB_SIZE_NAME)
 
             thumb = os.path.join(self.dir.path,
-                                 self._add_size_qualifier(img.filename, 'thumb'))
+                                 self._add_size_qualifier(img.filename,
+                                                          THUMB_SIZE_NAME))
             link_vals['thumb_width'],\
                 link_vals['thumb_height'] = img.get_size(thumb)
 
@@ -200,6 +224,7 @@ class WebalbumPage(make.FileMakeObject):
                 osize_info['link'] = self._add_size_qualifier(filename\
                                                               + '.html',
                                                               osize_name)
+                osize_info['link'] = self.url_quote(osize_info['link'])
             osize_index_links.append(osize_info)
 
         return osize_index_links
@@ -210,6 +235,9 @@ class WebalbumPage(make.FileMakeObject):
     def _do_not_escape(self, value):
         return genshi.core.Markup(value)
 
+    def url_quote(self, url):
+        return urllib.quote(url.encode(sys.getfilesystemencoding()), safe=':/')
+
 
 class WebalbumBrowsePage(WebalbumPage):
 
@@ -217,23 +245,26 @@ class WebalbumBrowsePage(WebalbumPage):
         self.image = image_file
         WebalbumPage.__init__(self, dir, size_name, self.image.name)
 
-        self.add_dependency(ImageOtherSize(self.dir,
-                                           self.image,
-                                           self.size_name))
+        if self.dir.album.browse_sizes[size_name] == (0, 0):
+            self.add_dependency(ImageOriginal(self.dir, self.image))
+        else:
+            self.add_dependency(ImageOtherSize(self.dir,
+                                               self.image,
+                                               self.size_name))
+            if self.dir.album.original and not self.dir.album.orig_base:
+                self.add_dependency(ImageOriginal(self.dir, self.image))
 
         # Depends on source directory in case an image was deleted
         self.add_dependency(self.dir.source_dir)
 
-        if self.dir.album.original:
-            self.add_dependency(ImageOriginal(self.dir, self.image))
-
         self.page_template = self.dir.album.templates['browse.thtml']
         self.add_file_dependency(self.page_template.path)
 
     def prepare(self):
         for prevnext in [self.image.previous_image, self.image.next_image]:
             if prevnext:
-                self.add_dependency(ImageOtherSize(self.dir, prevnext, 'thumb'))
+                self.add_dependency(ImageOtherSize(self.dir, prevnext,
+                                                   THUMB_SIZE_NAME))
 
     def build(self):
         self.dir.album.log(_("  XHTML %s") % os.path.basename(self.page_path),
@@ -278,7 +309,16 @@ class WebalbumBrowsePage(WebalbumPage):
         tpl_values['comment'] = self.image.info().get_comment()
 
         if self.dir.album.original:
-            tpl_values['original_link'] = self.image.filename
+            if self.dir.album.orig_base:
+                tpl_values['original_link'] = os.path.join(\
+                    self.dir.source_dir.rel_root(),
+                    self.dir.album.orig_base,
+                    self.dir.source_dir.strip_root(),
+                    self.image.filename)
+            else:
+                tpl_values['original_link'] = self.image.filename
+            tpl_values['original_link'] =\
+                self.url_quote(tpl_values['original_link'])
 
         self.page_template.dump(tpl_values, self.page_path)
 
@@ -311,7 +351,7 @@ class WebalbumIndexPage(WebalbumPage):
                 self.subdirs = []
 
         for image in self.images:
-            thumb_dep = ImageOtherSize(self.dir, image, 'thumb')
+            thumb_dep = ImageOtherSize(self.dir, image, THUMB_SIZE_NAME)
             self.add_dependency(thumb_dep)
             image_dep = WebalbumBrowsePage(self.dir, size_name, image)
             self.add_dependency(image_dep)
@@ -348,6 +388,7 @@ class WebalbumIndexPage(WebalbumPage):
                 filename = self._get_paginated_name(onum)
                 onum_info['link'] = self._add_size_qualifier(filename + '.html',
                                                              self.size_name)
+                onum_info['link'] = self.url_quote(onum_info['link'])
             onum_index_links.append(onum_info)
 
         return onum_index_links
@@ -359,13 +400,23 @@ class WebalbumIndexPage(WebalbumPage):
 
         values = {}
 
+        related_index_fn = self._add_size_qualifier(\
+                            WebalbumIndexPage.FILENAME_BASE_STRING + '.html',
+                            self.size_name)
+        if not self.dir.source_dir.is_album_root():
+            # Parent index link not for album root
+            values['parent_index_link'] = related_index_fn
+
         values['osize_index_links'] = self._get_osize_links(self._get_paginated_name())
         values['onum_index_links'] = self._get_onum_links()
 
         subgal_links = []
         for subdir in self.subdirs:
             dir_info = {'name': subdir.human_name,
-                        'link': subdir.source_dir.name + '/'}
+                        'link': '/'.join([subdir.source_dir.name,
+                                          related_index_fn]),
+                       }
+            dir_info['link'] = self.url_quote(dir_info['link'])
             dir_info.update(self.dir.metadata.get(subdir.source_dir.name))
             dir_info['album_picture'] = os.path.join(subdir.source_dir.name,
                                      self.dir.album.get_webalbumpic_filename())
@@ -387,7 +438,8 @@ class WebalbumIndexPage(WebalbumPage):
         values['title'] = self.dir.human_name
 
         if self.dir.album.dirzip and self.dir.dirzip:
-            values['dirzip'] = os.path.basename(self.dir.dirzip.path)
+            archive_name = os.path.basename(self.dir.dirzip.path)
+            values['dirzip'] = self.url_quote(archive_name)
 
         self.page_template.dump(values, self.page_path)
 
@@ -438,6 +490,12 @@ class LightWebalbumDir(make.FileMakeObject):
 
         self.dirzip = None
 
+    def get_all_images_count(self):
+        all_images_count = len(self.images_names)
+        for subdir in self.subdirs:
+            all_images_count += subdir.get_all_images_count()
+        return all_images_count
+
     def get_all_images_paths(self):
         all_images_paths = map(lambda fn: os.path.join(self.path, fn),
                                self.images_names)
@@ -496,7 +554,27 @@ class WebalbumDir(LightWebalbumDir):
             self.add_dependency(WebalbumArchive(self))
 
     def prepare(self):
-        self.images.sort(lambda x, y: x.compare_date_taken(y))
+        if self.album.subgal_sort_by[0] == 'mtime':
+            subgal_sorter = lambda x, y:\
+                                x.source_dir.compare_mtime(y.source_dir)
+        elif self.album.subgal_sort_by[0] == 'filename':
+            subgal_sorter = lambda x, y:\
+                                x.source_dir.compare_filename(y.source_dir)
+        else:
+            raise ValueError(_("Unknown sorting criterion '%s'")\
+                             % self.album.subgal_sort_by[0])
+        self.subdirs.sort(subgal_sorter, reverse=self.album.subgal_sort_by[1])
+
+        if self.album.pic_sort_by[0] == 'exif':
+            sorter = lambda x, y: x.compare_to_sort(y)
+        elif self.album.pic_sort_by[0] == 'mtime':
+            sorter = lambda x, y: x.compare_mtime(y)
+        elif self.album.pic_sort_by[0] == 'filename':
+            sorter = lambda x, y: x.compare_filename(y)
+        else:
+            raise ValueError(_("Unknown sorting criterion '%s'")\
+                             % self.album.pic_sort_by[0])
+        self.images.sort(sorter, reverse=self.album.pic_sort_by[1])
 
         # chain images
         previous_image = None
@@ -659,10 +737,14 @@ class SharedFiles(make.FileSimpleDependency):
 class Album:
 
     def __init__(self, source_dir, thumb_size, browse_sizes,
-                 quality=85, thumbs_per_page=0, dirzip=False):
+                 optimize=False, progressive=False,
+                 quality=85, thumbs_per_page=0, dirzip=False,
+                 pic_sort_by=('exif', False),
+                 subgal_sort_by=('filename', False)):
         self.set_logging()
 
         self.source_dir = os.path.abspath(source_dir)
+        self.source_dir = self.source_dir.decode(sys.getfilesystemencoding())
 
         self.thumb_size = thumb_size
         self.browse_sizes = dict(browse_sizes)
@@ -673,8 +755,17 @@ class Album:
         self.tpl_loader = None
         self.tpl_vars = {}
         self.original = False
+        self.orig_base = None
         self.thumbs_per_page = thumbs_per_page
         self.dirzip = dirzip
+        self.save_options = {}
+        if optimize:
+            self.save_options['optimize'] = True
+        if progressive:
+            self.save_options['progressive'] = True
+
+        self.pic_sort_by = pic_sort_by
+        self.subgal_sort_by = subgal_sort_by
 
     def set_theme(self, theme='default', default_style=None):
         self.theme = theme
@@ -722,8 +813,12 @@ class Album:
             styles.append(style)
         return styles
 
-    def set_original(self, original = False):
+    def set_original(self, original=False, orig_base=None):
         self.original = original
+        if self.original and orig_base:
+            self.orig_base = orig_base
+        else:
+            self.orig_base = None
 
     def set_webalbumpic(self, bg='transparent'):
         self.webalbumpic_bg = bg
@@ -760,12 +855,16 @@ class Album:
         if size_name == self.default_size_name and extension == '.html':
             # Do not append default size name to HTML page filename
             return path
+        elif size_name in self.browse_sizes.keys()\
+        and self.browse_sizes[size_name] == (0, 0) and extension != '.html':
+            # Do not append size_name to unresized images.
+            return path
         else:
             return "%s_%s%s" % (filename, size_name, extension)
 
     def _str_humanize(self, text):
         dash_replaced = text.replace('_', ' ')
-        return string.capwords(dash_replaced)
+        return dash_replaced
 
     def is_in_sourcetree(self, path):
         head = path
@@ -827,14 +926,19 @@ class Album:
 
             if dir_heap.has_key(root):
                 subdirs = dir_heap[root]
-                subdirs.sort(lambda x, y: cmp(x.source_dir.name,
-                                              y.source_dir.name))
                 del dir_heap[root] # No need to keep it there
             else:
                 subdirs = []
 
-            destgal = WebalbumDir(dir, subdirs, self, sane_dest_dir, clean_dest)
             light_destgal = LightWebalbumDir(dir, subdirs, self, sane_dest_dir)
+
+            if light_destgal.get_all_images_count() < 1:
+                self.log(_("(%s) and childs have no photos, skipped")
+                           % dir.path)
+                continue
+
+            destgal = WebalbumDir(dir, subdirs, self, sane_dest_dir, clean_dest)
+
             if not dir.is_album_root():
                 container_dirname = os.path.dirname(root)
                 if not dir_heap.has_key(container_dirname):
