@@ -32,7 +32,10 @@ class File(make.FileSimpleDependency):
         self.name, self.extension = os.path.splitext(self.filename)
 
     def _path_to_unicode(self, path):
-        return path.decode(sys.getfilesystemencoding())
+        if type(path) is unicode:
+            return path
+        else:
+            return path.decode(sys.getfilesystemencoding())
 
     def strip_root(self, path=None):
         found = False
@@ -81,6 +84,12 @@ class File(make.FileSimpleDependency):
                 return True
         return False
 
+    def compare_mtime(self, other_file):
+        return int(self.get_mtime() - other_file.get_mtime())
+
+    def compare_filename(self, other_file):
+        return cmp(self.filename, other_file.filename)
+
 
 class ImageFile(File):
 
@@ -101,6 +110,13 @@ class ImageFile(File):
         im = Image.open(img_path)
         return im.size
 
+    def has_exif_date(self):
+        exif_date = self.info().get_date()
+        if exif_date:
+            return True
+        else:
+            return False
+
     def get_date_taken(self):
         exif_date = self.info().get_date()
         if exif_date:
@@ -116,14 +132,34 @@ class ImageFile(File):
         delta = date1 - date2
         return int(delta)
 
+    def compare_no_exif_date(self, other_img):
+        # Comparison between 'no EXIF' and 'EXIF' sorts EXIF after.
+        if self.has_exif_date():
+            return 1
+        else:
+            return -1
+
+    def compare_to_sort(self, other_img):
+        if self.has_exif_date() and other_img.has_exif_date():
+            return self.compare_date_taken(other_img)
+        elif not self.has_exif_date() and not other_img.has_exif_date():
+            return self.compare_filename(other_img)
+        else:
+            # One of the picture has no EXIF date, so we arbitrary sort it
+            # before the one with EXIF.
+            return self.compare_no_exif_date(other_img)
+
 
 class Directory(File):
 
     def __init__(self, source, dirnames, filenames, album):
         File.__init__(self, self._path_to_unicode(source), album)
 
+        # No breaking up of filename and extension for directories
+        self.name = self.filename
+        self.extension = None
+
         self.dirnames = map(self._path_to_unicode, dirnames)
-        self.dirnames.sort()
         self.filenames = map(self._path_to_unicode, filenames)
 
     def guess_directory_picture(self, subdir = None):
